@@ -22,7 +22,7 @@
 // Local includes
 #include "messagewidget.hpp"
 #include "providers/providerinterface.hpp"
-#include "contacts.h"
+#include "providers/contacts.h"
 #include "ui_messagewidget.h"
 
 // Global includes
@@ -39,7 +39,6 @@ class MessageWidgetPrivate : public QObject
 
 public:
   MessageWidgetPrivate()
-  : provider(NULL)
   {}
 
   MessageWidget* self;
@@ -47,7 +46,8 @@ public:
   // Properties
   Ui_MessageWidget ui;
 
-  ProviderInterface* provider;
+  ProviderInfo providerInfo;
+  ProviderInterfacePtr provider;
 
   // Functions
   void init();
@@ -58,6 +58,7 @@ public slots:
   void onLogoutReply(bool success);
   //void onError(const QString& message);
   void onFreeSmsCountChanged(int count);
+  void onSendMessageReply(bool success, int messageId);
 
   // Ui callbacks
   void onChooseContactButtonClicked();
@@ -82,6 +83,9 @@ void MessageWidgetPrivate::onLoginReply(bool success)
 {
   // Enable or disable the send button
   ui.sendButton->setEnabled(success);
+
+  if ( !success )
+    QMessageBox::critical(self, self->windowTitle(), tr("Login failed. Please check your account data."));
 }
 
 //////////////////////////////////////////////////////////////////////////
@@ -115,35 +119,57 @@ void MessageWidgetPrivate::onFreeSmsCountChanged(int count)
 
 //////////////////////////////////////////////////////////////////////////
 
+void MessageWidgetPrivate::onSendMessageReply(bool success, int messageId)
+{
+  Q_UNUSED(messageId);
+
+  if ( !success )
+    return;
+
+  // Get the message text
+  QString message = ui.messageText->toPlainText();
+
+  // List all contacts and add the message to sent
+  foreach(const ContactInfo& contact, ui.contactEdit->contacts())
+  {
+    QString name = contact.name();
+    QString number = contact.number();
+    QString uid = contact.uniqueId();
+
+    // Really add the messages
+    add_to_sent_sms(&name, &number, &uid, &message);
+  }
+}
+
+//////////////////////////////////////////////////////////////////////////
+
 void MessageWidgetPrivate::onChooseContactButtonClicked()
 {
   QString dialogTitle = tr("Choose a contact");
-  QString name, number, uid;
+  ContactInfoList contacts;
 
   // Let the user choose a number
-  if ( get_contact_for_sms(&dialogTitle, &name, &number, &uid) )
+  if ( get_contact_for_sms(&dialogTitle, &contacts) )
   {
-    // Try to get the number in a international format
-    if ( !make_number_international(&number) )
-    {
-      QMessageBox::critical(self, tr("Wrong number format"),
-          tr("%1's phone number \"%2\" could not be converted to an international format.").arg(name).arg(number));
-
-      return; // There's no reason to continue
-    }
-
     // Get the current text
-    QString currentText = ui.contactEdit->text();
+    foreach(ContactInfo contact, contacts)
+    {
+      // Try to convert the number to an international format
+      QString number = contact.number();
+      if ( !make_number_international(number) )
+      {
+        QMessageBox::critical(self, tr("Wrong number format"),
+            tr("%1's phone number \"%2\" could not be converted to an international format.").arg(contact.name()).arg(number));
+      
+        continue;
+      }
 
-    // Append some separation if there's already some text
-    if ( !currentText.isEmpty() )
-      currentText += ", ";
+      // Set the contacts' number back
+      contact.setNumber(number);
 
-    // Append the selected name and number to the field
-    currentText += QString("%1 <%2>").arg(name).arg(number);
-
-    // Set the text back to the control
-    ui.contactEdit->setText(currentText);
+      // Add the contact
+      ui.contactEdit->addContact(contact);
+    }
   }
 }
 
@@ -157,17 +183,18 @@ void MessageWidgetPrivate::onSendButtonClicked()
   progressDialog->setRange(0, 0);
   progressDialog->show();
 
-  connect(provider, SIGNAL(sendMessageReply(bool, int)), progressDialog, SLOT(close()));
+  connect(provider.data(), SIGNAL(sendMessageReply(bool, int)), this, SLOT(onSendMessageReply(bool, int)));
+  connect(provider.data(), SIGNAL(sendMessageReply(bool, int)), progressDialog, SLOT(accept()));
 
-  provider->sendMessage( ui.contactEdit->text(), ui.messageText->toPlainText() );
+  provider->sendMessage( ui.contactEdit->contacts(), ui.messageText->toPlainText() );
 }
 
 //////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////
 
-MessageWidget::MessageWidget(QWidget* parent /* = 0 */)
-: QWidget(parent)
+MessageWidget::MessageWidget(QWidget* parent /* = 0 */, Qt::WindowFlags f /* = 0 */)
+: QWidget(parent, f)
 , d(new MessageWidgetPrivate)
 {
   d->self = this;
@@ -183,7 +210,29 @@ MessageWidget::~MessageWidget()
 
 //////////////////////////////////////////////////////////////////////////
 
-void MessageWidget::setProvider(ProviderInterface* provider)
+void MessageWidget::setProviderInfo(const ProviderInfo& providerInfo)
+{
+  // Already the same?
+//  if ( d->providerInfo == providerInfo )
+//    return;
+
+  // Take the data
+  d->providerInfo = providerInfo;
+
+  setWindowTitle( tr("Send SMS with %1").arg(providerInfo.visibleName) );
+  d->ui.providerImage->setPixmap( providerInfo.icon.pixmap(60, 60, QIcon::Normal, QIcon::On) );
+}
+
+//////////////////////////////////////////////////////////////////////////
+
+ProviderInfo MessageWidget::providerInfo() const
+{
+  return d->providerInfo;
+}
+
+//////////////////////////////////////////////////////////////////////////
+
+void MessageWidget::setProvider(const ProviderInterfacePtr& provider)
 {
   // Something changed?
   if ( d->provider == provider )
@@ -192,8 +241,8 @@ void MessageWidget::setProvider(ProviderInterface* provider)
   // Disconnect the old provider
   if ( d->provider )
   {
-    disconnect(d->provider, SIGNAL(loginReply(bool)), d, SLOT(onLoginReply(bool)));
-    disconnect(d->provider, SIGNAL(logoutReply(bool)), d, SLOT(onLogoutReply(bool)));
+    disconnect(d->provider.data(), SIGNAL(loginReply(bool)), d, SLOT(onLoginReply(bool)));
+    disconnect(d->provider.data(), SIGNAL(logoutReply(bool)), d, SLOT(onLogoutReply(bool)));
 
     // Set a null pixmap
     d->ui.providerImage->setPixmap( QPixmap() );
@@ -205,19 +254,18 @@ void MessageWidget::setProvider(ProviderInterface* provider)
   // Connect the new provider
   if ( d->provider )
   {
-    // Connect signals
-    connect(d->provider, SIGNAL(loginReply(bool)), d, SLOT(onLoginReply(bool)));
-    connect(d->provider, SIGNAL(logoutReply(bool)), d, SLOT(onLogoutReply(bool)));
-    connect(d->provider, SIGNAL(freeSmsCountChanged(int)), d, SLOT(onFreeSmsCountChanged(int)));
+    d->onFreeSmsCountChanged(-1);
 
-    // Change the ui to the new provider
-    d->ui.providerImage->setPixmap( d->provider->pixmap() );
+    // Connect signals
+    connect(d->provider.data(), SIGNAL(loginReply(bool)), d, SLOT(onLoginReply(bool)));
+    connect(d->provider.data(), SIGNAL(logoutReply(bool)), d, SLOT(onLogoutReply(bool)));
+    connect(d->provider.data(), SIGNAL(freeSmsCountChanged(int)), d, SLOT(onFreeSmsCountChanged(int)));
   }
 }
 
 //////////////////////////////////////////////////////////////////////////
 
-ProviderInterface* MessageWidget::provider() const
+ProviderInterfacePtr MessageWidget::provider() const
 {
   return d->provider;
 }
